from unittest import TestCase, skip

from pip2spack.core.pypi_package import PyPiPackage
from pip2spack.tests.test_data import test_data_ctypesgen as ready_packages


class Test_TestProblematicPackages(TestCase):
    def setUp(self) -> None:
        self.spack = PyPiPackage(ready_packages["ctypesgen"])

    def test_ctypesgen_should_be_categorized_as_not_source(self):
        self.assertFalse(self.spack.source, "Package should be tread as wheel package")

    @skip
    def test_ctypesgen_should_contains_only_wheel_version(self):
        self.assertEqual(len(self.spack.versions), 6)



from pip2spack.actions.action import Action


class ShowVersion(Action):
    ACTION = "version"
    PARAM_NAME = "ACTION"

    def fill_parser_arguments(self):
        pass

    def process_action(self, configuration):
        print("pip2spack: v1.0")



from dataclasses import dataclass
from os import getenv, listdir
from os.path import exists, join, relpath, abspath
from typing import AnyStr

from pip2spack.framework.constants import BUILTIN_REPOSITORY_REL_PATH
from pip2spack.framework.messages import Messages


@dataclass
class SpackRepository:
    _spack_root: str = getenv("SPACK_ROOT", None)
    _builtin_repository: str = f'{join(_spack_root, relpath(BUILTIN_REPOSITORY_REL_PATH))}' if _spack_root else ""

    @property
    def directory(self):
        return self._spack_root

    @directory.setter
    def directory(self, custom_spack_root):
        if not exists(custom_spack_root):
            raise
        self._spack_root = custom_spack_root

    @property
    def builtin_repository(self):
        return self._builtin_repository

    def list_all_packages(self):
        return listdir(self.builtin_repository)

    def list_py_packages(self):
        return [x for x in self.list_all_packages() if x.startswith("py-")]

    def exists(self, package_name: str, python_only: bool = True) -> bool:
        if python_only:
            return package_name in self.list_py_packages()
        else:
            return package_name in self.list_all_packages()

    def get_package_path(self, package_name: str) -> AnyStr:
        """Return absolute path to the package.py if exists in builtin repository"""
        if not package_name.startswith('py-'):
            package_name = "py-" + package_name
        package_py_path: str = join(*(f"{self.builtin_repository}", package_name, "package.py"))
        if not self.exists(package_name) or not exists(package_py_path):
            # TODO create a new one
            Messages.warn(f"Package {package_name} does not exist in the builtin repository")
            raise
        return abspath(package_py_path)



from argparse import ArgumentParser

from pip2spack.actions.create.create_action import CreateAction
from pip2spack.actions.dowload.download_action import DownloadAction
from pip2spack.actions.update.update_action import UpdateAction
from pip2spack.actions.version import ShowVersion
from pip2spack.framework.messages import Messages


class ActionDispatcher:
    ACTION_HANDLERS = [CreateAction, DownloadAction, ShowVersion, UpdateAction]

    def __init__(self):
        self.parser = ArgumentParser()
        subparsers = self.parser.add_subparsers()
        self.action_handlers = {action_handler.ACTION: action_handler(subparsers) for action_handler in
                                self.ACTION_HANDLERS}

    def process_application(self):
        configuration = self.parser.parse_args()
        try:
            self.action_handlers[configuration.ACTION].process_action(configuration)
        except AttributeError:
            Messages.clean("pip2spack ::")
            Messages.clean("quick tool for pip -> spack package conversion\n")
            Messages.clean("Choose operation:"
                           "\n\t\t - create -- for creation a new package"
                           "\n\t\t - update -- for update a existing package.py")
            Messages.clean("")
            Messages.info("All information are base on the pypi.org")



import os


def get_spack_repository() -> str:
    """Return a absolute path about spack repository"""
    if not os.getenv("SPACK_ROOT", False):
        print('Please provide a information about SPACK_ROOT')
        exit(1)

    repository_suburi = os.path.relpath('var/spack/repos/builtin/packages/')
    repository_full_path = os.path.join(os.getenv("SPACK_ROOT"), repository_suburi)

    if not os.path.exists(repository_full_path):
        print('Spack repository does not exists')
        exit(1)
    return repository_full_path


def create_directory(package_name):
    """Create a package directory for the provided package name"""
    if not package_name.startswith('py-'):
        package_name = 'py-' + package_name
    target_path = os.path.join(get_spack_repository(),
                               package_name)
    if os.path.exists(target_path):
        return False
    try:
        os.makedirs(target_path)
    except FileExistsError:
        pass
    finally:
        return os.path.exists(target_path)


def create_package(name, raw) -> str:
    if not name.startswith('py-'):
        name = 'py-' + name
    target_path = os.path.join(get_spack_repository(),
                               name)
    with open(os.path.join(target_path, 'package.py'), 'w') as f:
        f.write(raw)

    return os.path.join(target_path, 'package.py')



from pip2spack.actions.action_dispatcher import ActionDispatcher


def main():
    try:
        action_dispatcher = ActionDispatcher()
        action_dispatcher.process_application()
    except ImportError as err:
        print(err)
        exit(1)


if __name__ == '__main__':
    main()
